<?php

const CATALOGUE_BASE      = '/root/dp18/data/products';
const USER_CATALOGUE_BASE = '/root/dp18/data/products/user';

class Product {
    
    const UNMODIFIED = 0;
    const MODIFIED = 1;
    const DELETED = 3;
    const ADDED = 4;

    public $full_id;
    public $name;
    public $description;
    public $category;
    public $prices;
    // Serve per evitare di scrivere in 'user-edits' tutti i prodotti nel catalogo e dunque
    // limitarsi a solo quelli effettivamente modificati
    public $state;
    
    public static function makeNew($fullId) {
        $p = new Product($fullId);
        $p->state = self::ADDED;
        return $p;
    }

    public function __construct($fullId) {
        $this->full_id = $fullId;
        $this->state = self::UNMODIFIED;
    }

    public function product_id() {
        return substr($this->full_id, $this->id_separator() + 1);
    }

    public function catalogue_id() {
        return substr($this->full_id, 0, $this->id_separator());
    }

    public function setPrice($idx, $value) {
        // Modifico il prezzo solo se il valore passato è positivo
        // Imposto il flag di modificato a true (una volta sola) nel caso il prezzo sia effettivamente diverso
        if ($value > 0 && (!isset($this->prices[$idx]) || $value !== $this->prices[$idx])) {
            $this->prices[$idx] = $value;
            $this->setModified();
        }
    }

    public function thumbnail() {
        return CATALOGUE_BASE . DIRECTORY_SEPARATOR . $this->full_id . '.png';
    }

    public function hasThumbnail() {
        return is_readable($this->thumbnail());
    }

    public function image() {
        return CATALOGUE_BASE . DIRECTORY_SEPARATOR . $this->full_id . '_large.png';
    }

    public function hasImage() {
        return is_readable($this->image());
    }
    
    public function setModified() {
        if ($this->state == self::UNMODIFIED) {
            $this->state = self::MODIFIED;
        }
    }
    
    public function isDeleted() { return $this->state == self::DELETED; }
    
    public function isDirty() { return $this->state != self::UNMODIFIED; }
    
    public function setDeleted() {
        if ($this->state != self::ADDED) {
            $this->state = self::DELETED;
        }
    }
    
    public function undelete() {
        if ($this->state == self::DELETED) {
            $this->state = self::UNMODIFIED;
        }
    }

    private function id_separator() {
        if (FALSE !== ($slash = strpos($this->full_id, '/'))) {
            return $slash;
        } else {
            return -1;
        }
    }

}

function full_id($catalogueName, $id) {
    $slash = strpos($id, '/');
    if ($slash === FALSE) { // id relativo
        return "$catalogueName/$id";
    } else { // id completo
        return $id;
    }
}

function strattr($xmlElem, $attrName) {
    return isset($xmlElem[$attrName]) ? (string) $xmlElem[$attrName] : '';
}

function deserialize_catalogues(...$manifests) {
    $products = [];
    foreach ($manifests as $manifest) {
        $xmlCatalogue = simplexml_load_file($manifest);
        $catalogueName = (string) $xmlCatalogue['name'];
        foreach ($xmlCatalogue->product as $xmlProd) {
            $fullId = full_id($catalogueName, strattr($xmlProd, 'id'));
            // se il prodotto è presente lo modifico, altrimenti ne creo uno nuovo e lo aggiungo
            $p = isset($products[$fullId]) ? $products[$fullId] : ($products[$fullId] = new Product($fullId));

            if ($xmlProd->deleted) {
                $p->setDeleted();
            } else {
                if (isset($xmlProd['name'])) {
                    $p->name = $xmlProd['name'];
                }
                if (isset($xmlProd['category'])) {
                    $p->category = $xmlProd['category'];
                }
                if (isset($xmlProd->description)) {
                    $p->description = $xmlProd->description;
                }
                if (isset($xmlProd->prices->price)) {
                    foreach ($xmlProd->prices->price as $price) {
                        $p->prices[(int) $price['level']] = (int) $price;
                    }
                }
            }
        }
    }
    return $products;
}

function apply_modifications(&$products, $modPath) {
    if (!is_readable($modPath)) {
        return;
    }
    $xmlCatalogue = simplexml_load_file($modPath);
    // ignoro il nome del catalogo
    foreach ($xmlCatalogue->product as $xmlProd) {
        $fullId = strattr($xmlProd, 'id'); // nel file di modifiche ci devono essere ID completi
        // se il prodotto è presente lo modifico, altrimenti ne creo uno nuovo e lo aggiungo
        if (isset($products[$fullId])) {
            $p = $products[$fullId];
        } else {
            $p = Product::makeNew($fullId);
            $products[$fullId] = $p;
        }

        if (isset($xmlProd['deleted'])) {
            $p->setDeleted();
        } else {
            $p->setModified();
            if (isset($xmlProd['name'])) {
                $p->name = $xmlProd['name'];
            }
            if (isset($xmlProd['category'])) {
                $p->category = $xmlProd['category'];
            }
            if (isset($xmlProd->description)) {
                $p->description = $xmlProd->description;
            }
            if (isset($xmlProd->prices->price)) {
                foreach ($xmlProd->prices->price as $price) {
                    $p->prices[(int) $price['level']] = (int) $price;
                }
            }
        }
    }
}

function get_catalogue_manifests($catalogueDirPath) {
    $catalogueDir = dir($catalogueDirPath);
    $manifests = [];
    while (false !== ($entry = $catalogueDir->read())) {
        if ($entry !== '.' && $entry !== '..') {
            $manifests[] = join(DIRECTORY_SEPARATOR, [$catalogueDir->path, $entry, 'index.xml']);
        }
    }
    return $manifests;
}

function write_products_to_xml($xmlWriter, $products, $fullId = TRUE) {
    foreach ($products as $prodId => $prod) {
        $xmlWriter->startElement('product');
        $xmlWriter->writeAttribute('id', $fullId ? $prodId : $prod->product_id());
        //if ($prod->deleted) {
        if ($prod->isDeleted()) {
            $xmlWriter->writeAttribute('deleted', 'true');
        } else {
            $xmlWriter->writeAttribute('name', $prod->name);
            $xmlWriter->writeAttribute('category', $prod->category);
            $xmlWriter->writeElement('description', $prod->description);
            $xmlWriter->startElement('prices');
            foreach ($prod->prices as $idx => $price) {
                $xmlWriter->startElement('price');
                $xmlWriter->writeAttribute('level', $idx);
                $xmlWriter->text($price);
                $xmlWriter->endElement();
            }
            $xmlWriter->endElement(); // </prices>
        }
        $xmlWriter->endElement(); // </product>
    }
}

function generate_mod_file($modPath, $products) {

    function modified_products($prod) {
        return $prod->isDirty();
    }

    $out = new XMLWriter();
    $out->openUri('file://' . $modPath);
    $out->setIndent(TRUE);
    $out->setIndentString('  ');
    $out->startDocument('1.0', 'UTF-8');
    $out->startElement('catalogue');
    $out->writeAttribute('name', 'mod-file');
    write_products_to_xml($out, array_filter($products, "modified_products"), true);
    $out->endElement(); // </catalogue>
    $out->endDocument();
    $out->flush();
}
