<?php
require_once 'includes/access.php';

class Record {
    public $rowid;
    public $timestamp;
    public $category;
    public $data;
    
    function __construct($record) {
        $this->rowid = $record['rowid'];
        $this->timestamp = $record['timestamp'];
        $this->category = $record['category'];
        $this->data = $record['data'];
    }

    static $trCategory = [
        'info'                 => 'Informazione',
        'power-management'     => 'Alimentazione',
        'debug'                => 'Debug',
        'alarm'                => 'Allarme',
        'vending-session-info' => 'Vendita',
        'alarm-enable'         => 'Abilitazione tamper',
        'empty-dispenser'      => 'Prodotto esaurito',
        'empty-hopper'         => 'Hopper vuoto',
        'dispenser-error'      => 'Errore erogatore',
        'return-change'        => 'Resto erogato',
        'overpay'              => 'Extra incasso',
        'cash-inserted'        => 'Denaro inserito',
        'warning'              => 'Avviso',
        'support-request'      => 'Richiesta di assistenza'
    ];
    static $trAlarmCodes = [
        1 => 'Tamper', 2 => 'Funzionamento a batteria', 4 => 'Prodotti non disponibili', 8 => 'Impossibile immettere credito',
        16 => 'Touchscreen bloccato', 32 => 'Touchscreen non collegato', 64 => 'Scheda di controllo in errore'
    ];
    static $trPowerEvents = [
        'PowerOn' => 'Accensione',
        'PowerOff' => 'Spegnimento',
        'Restart' => 'Riavvio'
    ];
    
    public static function category_to_string($cat) { return isset(self::$trCategory[$cat]) ? self::$trCategory[$cat] : ''; }

    private static function money_str($amount) {
        return money_format('%n', $amount / 100);
    }

    private function render_row($data, $cls = "") {
        $displayCategory = self::category_to_string($this->category);
        $cssClass = empty($cls) ? $this->category : $cls;
        echo "<tr class='$cssClass'><td>$this->timestamp</td><td>$displayCategory</td><td>$data</td></tr>";
    }

    private function process_info() {
        $this->render_row($this->data, 'info');
    }

    private function process_power_mgmt() {
        $this->render_row(self::$trPowerEvents[$this->data], 'success power-management');
    }

    private function process_debug() {
        $this->render_row($this->data, 'active');
    }

    private function process_vending_session_info() {
        parse_str($this->data); // flag, cash, cashless
        $description = '';
        switch ($flag) {
            case 0: // inizio sessione
                $description = "Vendita iniziata"; break;
            case 1: // denaro prima di acquisto
                $cashValue =  money_str($cash);
                $cashlessValue =  money_str($cashless);
                $description = "Credito disponibile: contante $cashValue - cashless: $cashlessValue";
                break;
            case 2:
                $description = "Vendita conclusa";
        }
        $this->render_row($description, 'success');
    }

    private function process_alarm() {
        parse_str($this->data);
        $this->render_row(self::$trAlarmCodes[$id] . ($clear ? ' &mdash; rientrato' : ''), ($clear ? 'success' : 'danger'));
    }

    private function process_alarm_enable() {
        $this->render_row(((bool) $this->data) ? 'Abilitato' : 'Disinserito', 'info');
    }

    private function process_empty_dispenser() {
        parse_str($this->data); // productId, dispenser
        $dispenserId = $dispenser + 1;
        $this->render_row("Prodotto $productId esaurito alla selezione $dispenserId", 'warning');
    }

    private function process_empty_hopper() {
        $hopperId = $this->data + 1;
        $this->render_row("Hopper <b>$hopperId</b> vuoto", 'warning');
    }

    private function process_change() {
        $amount = self::money_str($this->data);
        $this->render_row("$amount euro", 'info');
    }

    private function process_overpay() {
        $amount = self::money_str($this->data);
        $this->render_row("$amount euro", 'warning');
    }

    private function process_cash_inserted() {
        parse_str($this->data); // banknotes, coins
        $banknotesValue = self::money_str($banknotes);
        $coinsValue = self::money_str($coins);
        $this->render_row("Banconote: $banknotesValue - Monete: $coinsValue", 'info');
    }

    private function process_warning() {
        $this->render_row($this->data, 'warning');
    }

    private function process_dispenser_error() {
        $dispenserId = $this->data + 1;
        render_row("Malfunzionamento erogatore $dispenserId", 'danger');
    }

    private function process_unknown() {
        $this->render_row($this->data);
    }

    public function render() {
        switch ($this->category) {
            case 'info': $this->process_info(); break;
            case 'power-management': $this->process_power_mgmt(); break;
            case 'debug': $this->process_debug(); break;
            case 'alarm': $this->process_alarm(); break;
            case 'vending-session-info': $this->process_vending_session_info(); break;
            case 'alarm-enable': $this->process_alarm_enable(); break;
            case 'empty-dispenser': $this->process_empty_dispenser(); break;
            case 'empty-hopper': $this->process_empty_hopper(); break;
            case 'dispenser-error': $this->process_dispenser_error(); break;
            case 'return-change': $this->process_change(); break;
            case 'overpay': $this->process_overpay(); break;
            case 'cash-inserted': $this->process_cash_inserted(); break;
            case 'warning': $this->process_warning(); break;
            case 'support-request': $this->render_row('Inoltrata richiesta di assistenza', 'info');
            default: $this->process_unknown();
        }
    }
}

class Pager {
    public $itemsPerPage;
    public $itemCount;
    public $currentPage;
    public $sideRadius;
    public $centerRadius;
    public $baseUrl;
    public $queryParams;
    
    public function __construct() {
        $this->itemsPerPage = 20;
        $this->itemCount = 0;
        $this->sideRadius = 2;
        $this->centerRadius = 3;
    }
    
    public function setItemCount($n) {
        $this->itemCount = $n;
    }
    
    public function setItemsPerPage($n) {
        $this->itemsPerPage = $n;
    }
    
    public function numPages() {
        return ceil($this->itemCount / $this->itemsPerPage);
    }
    
    public function setCurrentPage($n) {
        $this->currentPage = max(0, min($n, $this->numPages() - 1));
    }
    
    public function setBaseUrlAndQueryParams($url, $params) {
        $this->baseUrl = $url;
        $this->queryParams = $params;
    }
    
    public function offset() {
        return $this->currentPage * $this->itemsPerPage;
    }
    
    private function makePageUrl($page) {
        return $this->baseUrl . '?' . htmlentities(http_build_query(array_merge($this->queryParams, [ 'page' => $page, 'results' => $this->itemsPerPage ])));
    }
        
    private function previousLink() {
        if ($this->currentPage > 0) {
            $linkUrl = $this->makePageUrl($this->currentPage - 1);
            echo "<li><a href=\"$linkUrl#pages\">&laquo;</a></li>";
        } else {
            echo '<li class="disabled"><span>&laquo;</span></li>';
        }
    }
    
    private function pageLink($n) {
        $linkUrl = $this->makePageUrl($n);
        $pageIndex = $n + 1;
        $activeClass = $n === $this->currentPage ? 'class="active"' : '';
        echo "<li $activeClass><a href=\"$linkUrl#pages\">$pageIndex</a></li>";
    }
    
    private function nextLink() {
        if ($this->currentPage < $this->numPages() - 1) {
            $linkUrl = $this->makePageUrl($this->currentPage + 1);
            echo "<li><a href=\"$linkUrl#pages\">&raquo;</a></li>";
        } else {
            echo '<li class="disabled"><span>&raquo;</span></li>';
        }
    }
        
    public function renderLinks() {
?>
<nav>
    <ul class="pagination">
        <?php
        $this->previousLink();
        for ($i = 0; $i < $this->numPages(); ++$i) {
            $this->pageLink($i);
        }
        $this->nextLink();
        ?>
    </ul>
</nav>
<?php
    }
    
    public function renderLinksCompact() {
        $ellipsis = '<li class="disabled"><span>&hellip;</span></li>';
?>
<nav id="pages">
    <ul class="pagination">
        <?php
        $nPages = $this->numPages();
        $endLeft = $this->sideRadius;
        $winLeft = $this->currentPage - $this->centerRadius;
        $winRight = $this->currentPage + $this->centerRadius;
        $endRight = $this->numPages() - $this->sideRadius - 1;
        
        $this->previousLink();   
        for ($i = 0; $i <= min($endLeft, $winLeft - 1); ++$i) {
            $this->pageLink($i);
        }
        if ($winLeft > $endLeft + 1) {
            echo $ellipsis;
        }
        for ($i = max(0, $winLeft); $i <= min($winRight, $nPages - 1); ++$i) {
            $this->pageLink($i);
        }
        if ($endRight > $winRight + 1) {
            echo $ellipsis;
        }
        for ($i = max($endRight, $winRight + 1); $i < $nPages; ++$i) {
            $this->pageLink($i);
        }
        $this->nextLink();
        ?>
    </ul>
</nav>
<?php
    }
}


// parametri query nell'URL
$daysOld        = filter_input(INPUT_GET, 'days_old', FILTER_VALIDATE_INT, ['options' => ['min_range' => 0, 'default' => 7]]);
$category       = filter_var(filter_input(INPUT_GET, 'category', FILTER_SANITIZE_STRING)
                    , FILTER_VALIDATE_REGEXP, ['options' => ['regexp' => '/[A-Za-z-]+/', 'default' => '%']]);
$pageNum        = filter_input(INPUT_GET, 'page', FILTER_VALIDATE_INT, ['options' => ['min_range' => 0, 'default' => 0]]);
$resultsPerPage = filter_input(INPUT_GET, 'results', FILTER_VALIDATE_INT, ['options' => ['min_range' => 1, 'default' => 20]]);

$conn = new SQLite3('/root/dp18/log.db', SQLITE3_OPEN_READONLY);
$ageFilter  =  $daysOld > 0 ? "AND timestamp > datetime('now', '-$daysOld days', 'localtime')" : '';
$queryCount = "SELECT count(*) FROM Log WHERE category LIKE '$category' $ageFilter ORDER BY timestamp DESC";
$itemCount = $conn->query($queryCount)->fetchArray(SQLITE3_NUM)[0];
$pager = new Pager();
$pager->setItemCount($itemCount);
$pager->setItemsPerPage($resultsPerPage);
$pager->setCurrentPage($pageNum);
$pager->setBaseUrlAndQueryParams($_SERVER['PHP_SELF'], [ 'days_old' => $daysOld, 'category' => $category ]);
$query = "SELECT rowid, * FROM Log WHERE category LIKE '$category' $ageFilter ORDER BY timestamp DESC LIMIT $resultsPerPage OFFSET {$pager->offset()}";
$rowset = $conn->query($query);
$trCat = Record::category_to_string($category);
?>
<!DOCTYPE html>
<html>
    <head>
        <?php include 'templates/presentation_headers.php' ?>
        <title>Visualizzazione Log</title>
        <style type="text/css">
            //tr.info { background-color: lightblue }
            //tr.power-management { background-color: orange !important }
            tr.debug { background-color: lightgray }
            //tr.vending-session-info { background-color: lightgreen }
            //tr.alarm { background-color: lightcoral }
        </style>
    </head>
    <body>
        <?php include 'templates/navbar.php' ?>
        <div class="container">
            <div class="page-header"><h1>Visualizzazione Log</h1></div>
            <div class="panel panel-primary">
                <div class="panel-heading">
                    <h2 class="panel-title">
                        <a href="#filter-panel" data-toggle="collapse">
                            <span class="glyphicon glyphicon-filter"></span> Eventi degli ultimi <?= $daysOld ?> giorni <?= !empty($trCat) ? "nella categoria <i>$trCat</i>" : '' ?> (<?= $itemCount ?> risultati)
                        </a>
                    </h2>
                </div>
                <div class="collapse in" id="filter-panel">
                    <div class="panel-body">
                        <form class="form-horizontal" method="GET">
                            <div class="form-group">
                                <label class="col-sm-4 control-label">Giorni da visualizzare</label>
                                <div class="col-sm-8">
                                    <input class="form-control" type="number" name="days_old" min="0" value="<?= $daysOld ?>"/>
                                </div>
                            </div>
                            <div class="form-group">
                                <label class="col-sm-4 control-label">Risultati per pagina</label>
                                <div class="col-sm-8">
                                    <input class="form-control" type="number" name="results" min="1" value="<?= $resultsPerPage ?>"/>
                                </div>
                            </div>
                            <div class="form-group">
                                <label class="col-sm-4 control-label">Categoria</label>
                                <div class="col-sm-8">
                                    <select class="form-control" name="category">
                                        <option value="">Tutte</option>
                                        <?php
                                        foreach (Record::$trCategory as $cat => $repr) {
                                            $isSelectedAttribute = $cat === $category ? 'selected' : '';
                                            echo "<option value='$cat' $isSelectedAttribute>$repr</option>";
                                        }
                                        ?>
                                    </select>
                                </div>
                            </div>
                            <div class="form-group">
                                <div class="col-sm-offset-4 col-sm-8">
                                    <button type="submit" class="btn btn-primary">
                                        <span class="glyphicon glyphicon-filter"></span> Filtra
                                    </button>
                                </div>
                            </div>
                        </form>
                    </div>
                </div>
            </div>
            <?php $pager->renderLinksCompact() ?>
            <div class="table-responsive">
                <table class="table table-condensed table-hover">
                    <thead>
                        <tr>
                            <th>Timestamp</th>
                            <th>Categoria</th>
                            <th>Dettagli</th>
                        </tr>
                    </thead>
                    <tbody>
                        <?php
                        while ($row = $rowset->fetchArray(SQLITE3_ASSOC)) {
                            $record = new Record($row);
                            $record->render();
                        }
                        ?>
                    </tbody>
                </table>
            </div>
        </div>
        <?php include 'templates/footer.php' ?>
    </body>
</html>
<?php
$conn->close();